/* eslint-disable max-nested-callbacks */
require('./core.specs.js')

var chai = require('chai')

var expect = chai.expect,
  should = chai.should,
  assert = chai.assert

require('dotenv').config()

var init = require('../../utils/init')

const v3Url = {
  baseUrl: 'https://na1.api.riotgames.com/lol/match/v3/matchlists',
  postfix: '/by-account/47776491',
  query: '?queue=420&queue=440&champion=79'
}

const apiKeyPrefix = 'api_key='
const fakeKey = 'thisIsAKey'

const v3Opts = {
  foo: 'bar',
  arr: [1, 2, 3, 4, 5]
}

describe('Core Utils', function () {
  describe('makeUrl', function () {
    it('should make the correct url for v3', function () {
      const endUrl = 'summoner/v3/summoners/by-name/contractz'
      const testUrl = 'https://na1.api.riotgames.com/lol/summoner/v3/summoners/by-name/contractz'
      const url = init()._makeUrl(endUrl, 'na')
      assert.equal(url, testUrl)
    })
  })

  describe('setRegion', function () {
    describe('through init', function () {
      it('should throw on invalid region', function () {
        const api = require('../../dist/kindred-api')
        const debug = true
        const garbageRegion = 'foo'

        assert.throws(() => api.QuickStart(process.env.KEY, garbageRegion, true), Error)
      })

      it('should not throw on valid region', function () {
        assert.doesNotThrow(() => init(), Error)
      })
    })

    describe('through manual set', function () {
      it('should throw on invalid region', function () {
        const garbageRegion = 'north_amurica' // jokez!
        assert.throws(() => init().setRegion(garbageRegion), Error)
      })

      it('should not throw on valid region', function () {
        const k = init()
        const nonGarbageRegion = require('../../dist/kindred-api').REGIONS.KOREA

        assert.doesNotThrow(() => k.setRegion(nonGarbageRegion), Error)
      })
    })
  })

  describe('validName', function () {
    it('should return false', function () {
      const valid = init()._validName('foo%')
      assert.equal(valid, false)
    })

    it('should return true', function () {
      const valid = init()._validName('foo')
      assert.equal(valid, true)
    })

    describe('usage', function () {
      it('should throw on invalid name', function () {
        // name parameters -> valid name -> sanitize name -> throw
        const garbageName = 'foo%'
        assert.throws(() => init().Summoner.get(garbageName), Error)
      })

      it('should not throw on valid name 1', function () {
        // name parameters -> valid name -> sanitize name -> no throw
        assert.throws(() => init().Summoner.get('chauisthebest'), Error)
      })

      it('should not throw on valid name 2', function () {
        // name parameters -> valid name -> sanitize name -> no throw
        assert.throws(() => init().Summoner.get('chau.isthebest'), Error)
      })
    })
  })

  describe('sanitizeName', function () {
    it('should sanitize with valid name', function () {
      const name = init()._sanitizeName('foo Bar')
      assert.equal(name, 'foobar')
    })

    it('should throw with invalid name', function () {
      assert.throws(() => init()._sanitizeName('foo%'), Error)
    })
  })

  describe('stringifyOptions', function () {
    describe('v3', function () {
      const exp = 'foo=bar&arr=1&arr=2&arr=3&arr=4&arr=5'
      const actual = init()._stringifyOptions(v3Opts, v3Url.baseUrl + v3Url.postfix)
      assert.equal(exp, actual)
    })
  })

  describe('constructFullUrl', function () {
    describe('v3', function () {
      it('should work without params', function () {
        const url = v3Url.baseUrl + v3Url.postfix
        const marker = '?'
        const exp = url + marker + apiKeyPrefix + fakeKey
        const actual = init()._constructFullUrl(url, fakeKey)
        assert.equal(exp, actual)
      })

      it('should work with params', function () {
        const url = v3Url.baseUrl + v3Url.postfix + v3Url.query
        const marker = '&'
        const exp = url + marker + apiKeyPrefix + fakeKey
        const actual = init()._constructFullUrl(url, fakeKey)
        assert.equal(exp, actual)
      })
    })
  })

  describe('getAPIKeySuffix', function () {
    describe('w/o key', function () {
      describe('w/ `?`', function () {
        it('should work', function () {
          const k = init()
          const url = v3Url.baseUrl + v3Url.postfix + v3Url.query
          const exp = '&api_key='
          assert.equal(k._getAPIKeySuffix(url), exp)
        })
      })

      describe('w/o `?`', function () {
        it('should work', function () {
          const k = init()
          const url = v3Url.baseUrl + v3Url.postfix
          const exp = '?api_key='
          assert.equal(k._getAPIKeySuffix(url), exp)
        })
      })
    })

    describe('w/ key', function () {
      describe('w/ `?`', function () {
        it('should work', function () {
          const k = init()
          const url = v3Url.baseUrl + v3Url.postfix + v3Url.query
          const fakeKey = 'foo'
          const exp = '&api_key=' + fakeKey
          assert.equal(k._getAPIKeySuffix(url, fakeKey), exp)
        })
      })

      describe('w/o `?`', function () {
        it('should work', function () {
          const k = init()
          const url = v3Url.baseUrl + v3Url.postfix
          const fakeKey = 'foo'
          const exp = '?api_key=' + fakeKey
          assert.equal(k._getAPIKeySuffix(url, fakeKey), exp)
        })
      })
    })
  })
})